using System;
using System.Xml;
using System.Collections;

namespace com.ibm.gre
{
	/// <summary>
	/// Encapsulates a Records Manager Item List.
	/// Provides methods for getting attribute values.
	/// Also methods for removing specified Items. 
	/// </summary>
	public class IRMList
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="xmlList">A Records Manager List Item type XML string.
		/// See any Records Manager List method for an example of this type of XML.</param>
		/// <param name="strPrimaryKeyAttributeName">The name of the attribute that you wish to consider as the "Primary Key" for this List.
		/// The value of this attribute is returned by the PrimaryKey getter method.</param>
		/// <param name="strCaptionAttributeName">The name of the attribute that you wish to consider as the "Caption" for this lis.
		/// The value of this attribute is returned by the Caption getter method.</param>
		public IRMList(string xmlList, string strPrimaryKeyAttributeName, string strCaptionAttributeName)
		{
			m_strPrimaryKeyAttributeName = strPrimaryKeyAttributeName;
			m_strCaptionAttributeName = strCaptionAttributeName;
			
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xmlList);
			m_root = doc.DocumentElement;
			m_items = m_root.GetElementsByTagName("Item");
		}

		/// <summary>
		/// Returns the current value for the underlying XML for this List.
		/// </summary>
		public string XML
		{
			get
			{
				return m_root.OuterXml;
			}
		}

		/// <summary>
		/// Returns the number of Item elements in the List.
		/// </summary>
		public int Count
		{
			get
			{
				return m_items.Count;
			}
		}

		/// <summary>
		/// Returns the "Primary Key" value for the specified Item.
		/// The attribute that you wish to designate as the "Primary Key"
		/// is set in the constructor for this object.
		/// </summary>
		/// <param name="index">The index of specifying the Item to return the "Primary Key" for.</param>
		/// <returns>The value of the "Primary Key"</returns>
		public string PrimaryKey(int index)
		{
			return getVal(index, m_strPrimaryKeyAttributeName, "");
		}

		/// <summary>
		/// Returns the "Caption" value for the specified Item.
		/// The attribute that you wish to designate as the "Caption"
		/// is set in the constructor for this object.
		/// </summary>
		/// <param name="index">The index of specifying the Item to return the "Caption" for.</param>
		/// <returns>The value for the "Caption"</returns>
		public string Caption(int index)
		{
			return getVal(index, m_strCaptionAttributeName, "");
		}

		/// <summary>
		/// Gets the value for the named attribute at the specified index.
		/// </summary>
		/// <param name="index">The index of the Item to return the attribute value for.</param>
		/// <param name="attrName">The name of the attribute to return the value for.</param>
		/// <returns>Value of the specified attribute.</returns>
		public string getAttributeValue(int index, string attrName)
		{
			XmlNode node = m_items.Item(index).Attributes.GetNamedItem(attrName);
			if(node == null)
			{
				throw new Exception("Invalid Attribute: " + attrName);
			}
			return node.Value;
		}

		/// <summary>
		/// Removes the Item element at the specified index.
		/// </summary>
		/// <param name="index">The index of the Item element to remove.</param>
		public void removeItem(int index)
		{
			m_root.RemoveChild(m_items.Item(index));
		}

		private string getVal(int index, string attrName, string defaultVal)
		{
			XmlNode node = m_items.Item(index).Attributes.GetNamedItem(attrName);
			if(node != null)
			{
				string temp = node.Value;
				if(temp.Length > 0)
				{
					defaultVal = temp;
				}
			}
			return defaultVal;
		}

		private XmlNodeList m_items;
		private XmlElement m_root;
		private string m_strPrimaryKeyAttributeName;
		private string m_strCaptionAttributeName;
	}
}
