/*
 * Created on Jan 30, 2005
 */
package com.ibm.gre.samples.collectionhelpers;

import java.util.ArrayList;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;

import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.util.RecordManagerException;

/**
 * The IRMResultList encapsulates the XML returned from
 * a Records Manager Query.
 * The class contains methods for iterating through
 * all the results from a query while
 * iterating through all the individual Columns
 * for each result.
 * Tables of values can be built using this process.
 * There are methods that can be used to limit the number
 * of columns to display. 
 */
public class IRMResultList {
	
	/**
	 * Constructor
	 * @param irmQuery An IRMQuery object.
	 * This constructor is provide for future
	 * expansion of the class so that it may be possible
	 * to re-order column results for
	 * stateless clients.
	 * @throws RecordManagerException
	 * @throws Exception
	 */
	public IRMResultList(IRMQuery irmQuery) throws RecordManagerException, Exception 
	{
		m_SelectedColumns = new ArrayList();
		m_irmQuery = irmQuery;
		reLoad();
	}

	/**
	 * Constructor
	 * @param resultList
	 * The XML formatted String returned from
	 * the QueryControllerEJB.runQuery method.
	 * NOTE the queryResultOption value supplied
	 * to the the runQuery method must be the value
	 * com.ibm.gre.engine.util.QueryResultOptions.INCLUDE_COLUMNS
	 * so that the MetaData element is included
	 * in the result. 
	 * @throws RecordManagerException
	 */
	public IRMResultList(String resultList) throws RecordManagerException
	{
		m_SelectedColumns = new ArrayList();
		m_irmQuery = null;
		m_doc = XMLUtil.getDocument(resultList);
		Element root = m_doc.getDocumentElement();
		//(Element)XMLUtil.getNodeByXPath(m_rootReportTemplate,"BlockSize");
		m_MetaData = (Element)XMLUtil.getNodeByXPath(root,"MetaData");
		m_Data = (Element)XMLUtil.getNodeByXPath(root,"Data");
		m_Items = XMLUtil.getNodeListByXPath(m_Data,"./List/Item");
	}

	/**
	 * This method specifies the name of a column
	 * to include in the displayed results.
	 * If no names are specified the class will
	 * include all the columns specified in the
	 * XML used to construct this object.
	 * @param strFlPlnCmpntAttrDefColNm
	 * The File Plan Component Attribute Column Name
	 * that should be displayed.
	 * @throws Exception
	 */
	public void AddColumn(String strFlPlnCmpntAttrDefColNm) throws Exception
	{
		boolean ok = false;
		NodeList items = XMLUtil.getNodeListByXPath(m_MetaData,"./List/Item");
		for(int i = 0; i < items.getLength(); i++)
		{
			String test = XMLUtil.getValue(items.item(i).getAttributes().getNamedItem("FlPlnCmpntAttrDefColNm"));
			if(strFlPlnCmpntAttrDefColNm == test)
			{
				Column col = new Column(items.item(i));
				m_SelectedColumns.add(col);
				ok = true;
				break;
			}
		}
		if(!ok)
		{
			throw new Exception("The FlPlnCmpntAttrDefColNm: " + strFlPlnCmpntAttrDefColNm + " does not exist in the Result List MetaData");
		}
	}

	/**
	 * The number of columns to be displayed.
	 * Note this is either the number of columns
	 * that are specified using the above AddColumn
	 * method or all of the columns in the original
	 * XML if the AddColumn method is never called. 
	 * @return int
	 * @throws Exception
	 */
	public int getColumnCount() throws Exception
	{
		makeColumnsArray();
		return m_SelectedColumns.size();
	}

	/**
	 * Gets the Caption of a table column
	 * for the specified index.
	 * @param index
	 * The index into the collection of Columns for this result list.
	 * @return The Caption to display in the table header.
	 * @throws Exception
	 */
	public String getFlPlnCmpntAttrDefCapt(int index) throws Exception
	{
		makeColumnsArray();
		Column col = (Column)m_SelectedColumns.get(index);
		return col.getFlPlnCmpntAttrDefCapt();
	}

	/**
	 *  Gets the FlPlnCmpntAttrDefColNm of a table
	 * column for the specified index.
	 * This value is used in the getItemByFlPlnCmpntAttrDefColNm
	 * method to specify which value to return.
	 * @param index
	 * The index into the collection of Columns for this result list.
	 * @return The FlPlnCmpntAttrDefColNm to use to get
	 * the values for the column value.
	 * @throws Exception
	 */
	public String getFlPlnCmpntAttrDefColNm(int index) throws Exception
	{
		makeColumnsArray();
		Column col = (Column)m_SelectedColumns.get(index);
		return col.getFlPlnCmpntAttrDefColNm();
	}

	/**
	 * The suggested percentage of the total width for all Columns
	 * that the specified column should take up.
	 * @param index
	 * The index into the collection of Columns for this result list.
	 * @return Percentage Width
	 */
	public int getPercentageColumnWidth(int index)
	{
		makeColumnWidths();
		Column col = (Column)m_SelectedColumns.get(index);
		return col.getPercentWidth();
	}

	/**
	 * The Number of rows that will be returned in the result set.
	 * @return int
	 */
	public int getItemCount()
	{
		return m_Items.getLength();
	}

	/**
	 * This method gets the value for the column as specified
	 * by the strFlPlnCmpntAttrDefColNm param for the result set
	 * row specified by the index parameter.
	 * You fill table values by iterating through all
	 * index values from 0 to ItemCount.
	 * For each row you then iterate through all columns
	 * and call the getFlPlnCmpntAttrDefColNm method
	 * for each column value from 0 to ColumnCount to get
	 * the value to supply for the strFlPlnCmpntAttrDefColNm param.
	 * @param index Use this value to specify the index value of
	 * the Item (row) in the result set to return the value for.
	 * @param strFlPlnCmpntAttrDefColNm Use this value to specify the
	 * name of the column to return the result for.
	 * @return Returns the value of the named XML attribute in the specified position.
	 * @throws Exception
	 */
	public String getItemByFlPlnCmpntAttrDefColNm(int index, String strFlPlnCmpntAttrDefColNm)
	throws Exception
	{
		Attr attr = (Attr)m_Items.item(index).getAttributes().getNamedItem(strFlPlnCmpntAttrDefColNm);
		if(attr == null)
		{
			return "";
		}
		return XMLUtil.getValue(attr);
	}

	/**
	 * This is a variation on the above method.
	 * The FlPlnCmpntAttrDefColNm does not have to be known
	 * only the index of the column in the table.
	 * @param row Use this value to specify the index value of
	 * the Item (row) in the result set to return the value for.
	 * @param column Use this value to specify the index value of the column
	 * in the table.
	 * @return Returns the value of the XML attribute in the specified position.
	 * @throws Exception
	 */
	public String getItemByRowColumnIndex(int row, int column) throws Exception
	{
		Column col = (Column)m_SelectedColumns.get(column);
		Attr attr = (Attr)m_Items.item(row).getAttributes().getNamedItem(col.getFlPlnCmpntAttrDefColNm());
		if(attr == null)
		{
			return "";
		}
		return XMLUtil.getValue(attr);
	}

	/**
	 * Set the wheight value that is used for
	 * BINARY and BOOLEAN values when the
	 * percentage width is calculated for these types.
	 * The default value is 1.
	 * @param val
	 */
	public void setWheight1(byte val)
	{
		weight1 = val;
	}

	/**
	 * Set the wheight value that is used for
	 * DATE, DOUBLE, INTEGER and VIRTUAL values when the
	 * percentage width is calculated for these types.
	 * The default value is 2.
	 * This means a DATE value will take up twice as
	 * much column width as a BOOLEAN
	 * @param val
	 */
	public void setWheight2(byte val)
	{
		weight2 = val;
	}

	/**
	 * Set the wheight value that is used for
	 * DATETIME, CLOB and STRING values when the
	 * percentage width is calculated for these types.
	 * The default value is 3.
	 * This means a DATE value will take up three times as
	 * much column width as a BOOLEAN
	 * @param val
	 */
	public void setWheight3(byte val)
	{
		weight3 = val;
	}

	private void reLoad() throws RecordManagerException, Exception
	{
		m_doc = XMLUtil.getDocument(m_irmQuery.getResultList());
		Element root = m_doc.getDocumentElement();
		m_MetaData = (Element)XMLUtil.getNodeByXPath(root,"MetaData");
		m_Data = (Element)XMLUtil.getNodeByXPath(root,"Data");
		m_Items = XMLUtil.getNodeListByXPath(m_Data,"./List/Item");
	}

	private void makeColumnsArray() throws Exception
	{
		//If the developer has not specified a set of columns then
		//use all the columns available from the MetaData
		if(m_SelectedColumns.size() == 0)
		{
			NodeList items = XMLUtil.getNodeListByXPath(m_MetaData,"./List/Item");
			for(int i = 0; i < items.getLength(); i++)
			{
				Column col = new Column(items.item(i));
				m_SelectedColumns.add(col);
			}
		}
	}

	//This is a convenience method to determine the percentage widths of the displayed columns
	//Using the specified wheightings the defaults being bool = 1, numeric = 2, String = 3
	private void makeColumnWidths()
	{
		if(!widthsDone)
		{
			float totalWeights = 0;
			widthsDone = true;
			for(int i = 0; i < m_SelectedColumns.size(); i++)
			{
				totalWeights += wheightByFlPlnCmpntAttrDefDatTyp(((Column)m_SelectedColumns.get(i)).getFlPlnCmpntAttrDefDatTyp());	
			}
			for(int i = 0; i < m_SelectedColumns.size(); i++)
			{
				float weight = wheightByFlPlnCmpntAttrDefDatTyp(((Column)m_SelectedColumns.get(i)).getFlPlnCmpntAttrDefDatTyp());
				int percentWidth =  (int)((weight/totalWeights) * 100);
				((Column)m_SelectedColumns.get(i)).setPercentWidth(percentWidth);
			}
		}
	}

	private int wheightByFlPlnCmpntAttrDefDatTyp(int nFlPlnCmpntAttrDefDatTyp)
	{
		switch(nFlPlnCmpntAttrDefDatTyp)
		{
			case AttributeDataTypes.BINARY:
				return weight1;
			case AttributeDataTypes.BOOLEAN:
				return weight1;
			case AttributeDataTypes.CLOB:
				return weight3;
			case AttributeDataTypes.DATE:
				return weight2;
			case AttributeDataTypes.DATETIME:
				return weight3;
			case AttributeDataTypes.DOUBLE:
				return weight2;
			case AttributeDataTypes.INTEGER:
				return weight2;
			case AttributeDataTypes.STRING:
				return weight3;
			case AttributeDataTypes.VIRTUAL:
				return weight2;
			default:
				return weight2;
		}
	}

	protected NodeList m_Items;

	private IRMQuery m_irmQuery;
	private Document m_doc;
	private Element m_MetaData;
	private Element m_Data;
	private ArrayList m_SelectedColumns;
	private boolean widthsDone = false;
	private byte weight1 = 1;
	private byte weight2 = 2;
	private byte weight3 = 3;

}
